/*
 * (C) Copyright Pattern Recognition Group Delft. All rights reserved.
 *
 * STATLIB.H : Declarations of the general routines available from
 *              the statistical package.
 *
 * Made by : W.F. Schmidt
 * SccsId  : %Z%$RCSfile: statlib.h,v $ V%Q% $Revision: 1.16.1.6 $ $Date: 1997/04/17 12:42:05 $
 */
 
/* stat/cluspao.c */
int 			cluster_pao_init 		(DATASET *dset);
DATASET  *cluster_pao 				(double threshold_local);

/* stat/dblexp.c */
double 		random_dbl_exp 			(double a, double b);
double 		pr_dbl_exp 					(double x, double a, double b);
double 		pr_cum_dbl_exp 			(double x, double a, double b);

/* stat/dparzen.c */
int 			parzen_dataset_class 
															(DATASET *dset, SAMPLE *samp, double *S, double *P, int NCls);
double 		parzen_dset_best_s 	(DATASET *dset, int NCls, int class, double init_s);

/* stat/estdset.c */
int 			mean_var_dataset 		(DATASET *dset, int class, double *mean, double **var);

/* stat/esterror.c */
int 			add_classerror 			(double *pattern, int classA, int class, double *W, double b, int n);
double 		calc_classerror 		(void);

/* stat/estimat.c */
int 			add_mean_var 				(double *pattern, int n);
int 			calc_mean_var 			(int n, double *m, double **c);

/* stat/estqerror.c */
int 			addq_classerror 		(double *pattern, int classA, int class, double **W2, double *W1, double b, int n);
double 		calcq_classerror 		(void);

/* stat/fisher.c */
int 			fisher 							(double **Ka, double **Kb, double *Ma, double *Mb, double c, int n, double *W, double *b);
int 			fisher_dataset 			(DATASET *lset, int ClassA, int ClassB, double c, double *W, double *b);

/* stat/gasdev.c */
double 		ran1 								(int idum);
double 		gasdev 							(int idum);
int 			random_seed 				(int seed);

#define 	random_unif() 			ran1(0)
#define 	random_gauss() 			gasdev(0)

/* stat/gasmult.c */
int 			random_multi_init 	(double **cov, int n);
void 			random_multi_free 	(void);
int 			random_multi_gauss 	(double *mean, int n, double *pattern);

/* stat/higherr.c */
double 		highley_error 			(double c1, double c2, double c3);

/* stat/highley.c */
void 			random_highley_a 		(double *pattern);
void 			random_highley_b 		(double *pattern);
DATASET  *random_highley_dataset 
															(int NClassA, int NClassB);

/* stat/isodata.c */
DATASET  *isodata_basic 			(DATASET *dset, double DesiredMse);

/* stat/klexpan.c */
int 			kl_coordinates 			(DATASET *dset, double **v, double *d);

/* stat/knn.c */
int 			knn_init 						(DATASET *dset);
void 			knn_free 						(void);
int 			knn_classify 				(int k, int class, double *pattern);
int 			knn_majority 				(int k, double *pattern);
double 		knn_loo_perf 				(int k);

/* stat/maha.c */
int 		 *GetLabels 					(DATASET *Dset);
int 			mahal_multiclass_dataset 
															(DATASET *Dset, int *ClassL, double ***W2, double **W1, double *W0);
int 			Calc_Dist_Class 		(double *Pattern, int *label, double ***Wa2, double **Wa1, double *Wa0, double *Dist, int *Class, int No, int n);
double 		calc_multiclass_error 
															(DATASET *lset, DATASET *tset, int trace, int function);

/* stat/parzen.c */
int 			parzen_init 				(DATASET *dset);
void 			parzen_free 				(void);
double 		parzen_probability 	(double *pattern, int Label, double S);
int 			parzen_classify 		(double *pattern, double S0, double S1, double C);
int 			parzen_mclass 			(double *pattern, double *S, double *P);
double 		parzen_best_s 			(int class, double init_s);

/* stat/patnn.c */
double 		sample_distance 		(SAMPLE *sample1, SAMPLE *sample2, int dim);
int 			nearest_neighbour		(SAMPLE *sample1, DATASET *dataset);
double 		nn_performance 			(DATASET *dataset);
DATASET  *compute_means 			(DATASET *dataset);
double 		rss_min_dist_perf 	(DATASET *dataset);
double 		loo_min_dist_perf 	(DATASET *dataset);

/*
 * Perceptron stuff.
 */

#define FIXED_UPD   0x0
#define BOUND_UPD   0x1
#define NO_NORM_UPD 0x0
#define NORM_UPD    0x2
#define SYNC_UPD    0x0
#define ASYNC_UPD   0x4

typedef struct perceptron
{
	char *name;
	int nin;
	int nout;
	int nlearn;
	int mode;
	double (*f)();
	double (*df)();

	union w_tag 
	{
		double *vect;
		double **mat;
	} w;
	union dw_tag
	{
		double *vect;
		double **mat;
	} dw;
} PTRON;

/* stat/pctrgen.c */
int 			learn_ptron_init 		(DATASET *dset);
void 			learn_ptron_free 		(DATASET *dset);
int 			learn_ptron 				(PTRON *ptron, double c, int itt);
int 			eval_error_ptron 		(PTRON *ptron, double *error);

/* stat/pctrutil.c */
PTRON 	 *malloc_ptron 				(char *name, int nin, int nout, double (*f)(), double (*df)(), int mode);
void 			free_ptron 					(register PTRON *ptr);
void 			eval_ptron 					(PTRON *ptron, double *in, double *out);
int 			fprintf_ptron 			(FILE *stream, PTRON *ptron);

/* stat/prerf.c */
double 		pr_normal 					(double x);
double 		pr_k_normal 				(double *x, int k, double *mu, double **cinv, double detcinv);
double 		pr_cum_normal 			(double x);
double 		pr_error 						(double K, double M, double b);
double 		pr_error_multi 			(double **K, double *M, double *W, double b, int n);

/* stat/quadr.c */
int 			fisherq 						(double **Ka, double **Kb, double *Ma, double *Mb, double c, int n, double **W2, double *W1, double *b);
int 			fisherq_dataset 		(DATASET *lset, int ClassA, int ClassB, double c, double **W2, double *W1, double *b);

/* stat/sammon.c */

#define 	SAMMON_GD       			(0x01)
#define 	SAMMON_PN       			(0x02)

#define 	SAMMON_VERBOSE  			(0x01)
#define 	SAMMON_GRAPHICS 			(0x02)
#define 	SAMMON_NO_VALIDATION	(0x04)

#define		SAMMON_INPUT					(0x01)
#define		SAMMON_OUTPUT					(0x02)

double		euclidian_distance	(double *a, double *b, int n);
int 			sammon_init				 	(DATASET *dset, int dim);
int 			sammon_exit 				(void);
int 			sammon_grid 				(int dim);
double 		sammon_stress 			(void);
int 			sammon_dstress 			(int i, double *dE, double *d2E);
int 			sammon_plot 				(long loops);
int 			sammon_solve 				(int method, double learn_rate, double momentum, long max_steps, long step, int options);
DATASET  *sammon_extract_map 	(int mode);
void			sammon_inject_map		(DATASET *dmap, int mode);
double		sammon_constant 		(void);
void 			sammon_triangulate 	(int n1, int n2, int n3, double *vector, double *mapping);
void 			sammon_triangulate_set 	
															(DATASET *dset);

/* stat/scatter.c */
int 			all_scatter_dataset (DATASET *dset, double **St, double **Sw, double **Sb);

/* stat/sdaterr.c */
double 		varset_error 				(double c1, double c2, double c3);
double 		meanset_error 			(double c1, double c2, double c3);
double		duinset_error 			(int k, double *w, double c);
double 		duinset_overlap 		(int k);
double 		kvar_overlap 				(int k);
double 		khighley_overlap 		(int k);

/* stat/sdatset.c */
void 			random_varset_a 		(double *pattern);
void 			random_varset_b 		(double *pattern);
void 			random_meanset_a 		(double *pattern);
void 			random_meanset_b 		(double *pattern);
void 			random_duinset_a 		(register double *pattern, int k);
void 			random_duinset_b 		(register double *pattern, int k);
void 			random_kvar_a 			(register double *pattern, int k);
void 			random_kvar_b 			(register double *pattern, int k);
void 			random_khighley_a 	(double *x, int k);
void 			random_khighley_b 	(double *x, int k);
DATASET  *std_var_dataset 		(int NClassA, int NClassB);
DATASET  *std_mean_dataset 		(int NClassA, int NClassB);
DATASET  *duin_dataset 				(int NClassA, int NClassB, int k);
DATASET  *kvar_dataset 				(int NClassA, int NClassB, int k);
DATASET  *khighley_dataset 		(int NClassA, int NClassB, int k);


/*
 * End of incude file
 */

